//*******************************************************************//
//                                                                   //
//   - More Advanced Programming Topics -                            //
//                                                                   //
//   $Author:       TGraupmann         tgraupmann@yahoo.com          //
//                                                                   //
//   $Program:	3DS Animation PART 3                                  //
//                                                                   //
//   $Description:	Shows you how to read the .3DS object hierarchy	//
//                                                                   //
//   $Date:	06/01/03                                                 //
//                                                                   //
//*********************************************************************

#include "main.h"
#include "ui.h"

// Global variables are now contained in this UI structure
GTUI_Interface GTUI;


/////////////////////////////// InitGTTextures \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This initializes the textures of the 3DS model.
/////
/////////////////////////////// InitGTTextures \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
GLvoid InitGTTextures(t3DModel* pModel)
{
	static s_TextureCount = 0;
	// Go through all the materials
	for(int i = 0; i < pModel->numOfMaterials; i++)
	{
		// Check to see if there is a file name to load in this material
		if(strlen(pModel->pMaterials[i].strFile) > 0)
		{
			// Texture images are now expected to be in the same path as
			// the 3DS file.

			char strImagePath[255] = {0};
			char strImageFileName[255] = {0};
			for(int j = strlen(pModel->m_FileName)-1; j >= 0; j--)
			{
				if((pModel->m_FileName[j] == '/') ||
					(pModel->m_FileName[j] == '\\'))
				{
					strncpy(strImagePath, pModel->m_FileName, j+1);
					break;
				}
			}			
			sprintf(strImageFileName, "%s%s", strImagePath, pModel->pMaterials[i].strFile);
			// Use the name of the texture file to load the bitmap, with a texture ID (i).
			// We pass in our global texture array, the name of the texture, and an ID to reference it.	
			CreateTexture(GTUI.Texture, strImageFileName, s_TextureCount);
			s_TextureCount++;
		}

		// Set the texture ID for this material
		pModel->pMaterials[i].texureId = s_TextureCount-1;
	}
}


///////////////////////////////// InitLights \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This initializes OpenGL-Lighting
/////
///////////////////////////////// InitLights \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void InitLights(void)
{
   // * * * * * * * ****** //// TEMPORARY LIGHT VARIABLES \\\\\ ****** * * * * *
   // Light0
	GLfloat LightAmbient0[] = { 0.2f, 0.2f, 0.2f, 0.2f}; // Ambient Light Is 20% White
	GLfloat LightDiffuse0[] = { 0.4f, 0.4f, 0.4f, 1.0f}; // Diffuse Light Is White
	GLfloat LightSpecular0[] = { 0.8f, 0.8f, 0.8f, 0.8f}; // Diffuse Light Is White
	GLfloat LightPosition0[] = { -10, 40, 5}; // Position Is High Above	

	// Here, we turn on a lighting and enable lighting.  We don't need to
	// set anything else for lighting because we will just take the defaults.
	// We also want color, so we turn that on
	if(GTUI.LightingToggle)
	   glEnable(GL_LIGHTING); // Turn on lighting
	else
	{
      glDisable(GL_LIGHTING); // Turn off lighting
	}
	glEnable(GL_COLOR_MATERIAL);					// Allow color

	// Override defaults
	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient0); // Load Light-Parameters into GL_LIGHT1
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse0);
	glLightfv(GL_LIGHT0, GL_SPECULAR, LightSpecular0);
	glLightfv(GL_LIGHT0, GL_POSITION, LightPosition0);
	glEnable(GL_LIGHT0);
}


void InitDisplayList(t3DModel* pModel)
{
	int numOfObjects = pModel->pObject.size();
	for(int i=0; i<numOfObjects; i++)
	{
		// Get a pointer to each object
		t3DObject *pObject = &pModel->pObject[i];
		
		// Initialize Display Lists
		pModel->CreateDisplayList(pObject, GTUI.TotalDisplayLists + 1);
		GTUI.TotalDisplayLists++;
	}
}

///////////////////////////////// INIT GAME WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function initializes the game window.
/////
///////////////////////////////// INIT GAME WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void Init(HWND hWnd)
{
	GLWin.g_hWnd = hWnd;										// Assign the window handle to a global window handle
	GetClientRect(GLWin.g_hWnd, &GLWin.g_rRect);				// Assign the windows rectangle to a global RECT
	InitializeOpenGL(GLWin.g_rRect.right, GLWin.g_rRect.bottom); // Init OpenGL with the global rect

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);	// default clear color

	initGTUI();	

	for(int ModelCount=0; ModelCount<GTUI.ModelCount; ModelCount++)
	{
		// Get a pointer to each model
		t3DModel *pModel = &GTUI.c3DModel[ModelCount];

		// Initialize Textures
		InitGTTextures(pModel);

		// Compile Display Lists
		InitDisplayList(pModel);		
	}

	InitLights(); // Initialize OpenGL Lighting
}


///////////////////////////////// CREATE TEXTURE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This creates a texture in OpenGL that we can texture map
/////
///////////////////////////////// CREATE TEXTURE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void CreateTexture(UINT textureArray[], LPSTR strFileName, int textureID)
{
	AUX_RGBImageRec *pBitmap = NULL;
	
	if(!strFileName)									// Return from the function if no file name was passed in
		return;

	FILE *fCheck = 0;
	
	if( (fCheck = fopen(strFileName, "rct")) == NULL)  
		pBitmap = auxDIBImageLoad("data/default.bmp");
	else
	{
		fclose(fCheck);
		pBitmap = auxDIBImageLoad(strFileName);		// Load the bitmap and store the data
	}
	
	if(pBitmap == NULL) // Load our default texture if a problem occurs
		pBitmap = auxDIBImageLoad("data/default.bmp");
		//exit(0);

	// Generate a texture with the associative texture ID stored in the array
	glGenTextures(1, &textureArray[textureID]);

	// This sets the alignment requirements for the start of each pixel row in memory.
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	// Bind the texture to the texture arrays index and init the texture
	glBindTexture(GL_TEXTURE_2D, textureArray[textureID]);

	// Build Mipmaps (builds different versions of the picture for distances - looks better)
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, pBitmap->sizeX, pBitmap->sizeY, GL_RGB, GL_UNSIGNED_BYTE, pBitmap->data);

	// Lastly, we need to tell OpenGL the quality of our texture map.  GL_LINEAR is the smoothest.
	// GL_NEAREST is faster than GL_LINEAR, but looks blochy and pixelated.  Good for slower computers though.
	// Read more about the MIN and MAG filters at the bottom of main.cpp
		
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	

	// Now we need to free the bitmap data that we loaded since openGL stored it as a texture

	if (pBitmap)										// If we loaded the bitmap
	{
		if (pBitmap->data)								// If there is texture data
		{
			free(pBitmap->data);						// Free the texture data, we don't need it anymore
		}

		free(pBitmap);									// Free the bitmap structure
	}
}

/////////////////////////////////////////////////////////////////////////////////
///////////////////////////////WIN32 WINDOW HANDLING\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Code to follow is explicitly for Win32 Window Handling
/////
///////////////////////////////WIN32 WINDOW HANDLING\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////////////////////////////////////////////////////////////////////////////////

GLWindow GLWin;


///////////////////////////////// CHANGE TO FULL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This changes the screen to FULL SCREEN
/////
///////////////////////////////// CHANGE TO FULL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void ChangeToFullScreen()
{
	DEVMODE dmSettings;									// Device Mode variable

	memset(&dmSettings,0,sizeof(dmSettings));			// Makes Sure Memory's Cleared

	// Get current settings -- This function fills our the settings
	// This makes sure NT and Win98 machines change correctly
	if(!EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&dmSettings))
	{
		// Display error message if we couldn't get display settings
		MessageBox(NULL, "Could Not Enum Display Settings", "Error", MB_OK);
		return;
	}

	dmSettings.dmPelsWidth	= SCREEN_WIDTH;				// Selected Screen Width
	dmSettings.dmPelsHeight	= SCREEN_HEIGHT;			// Selected Screen Height
	
	// This function actually changes the screen to full screen
	// CDS_FULLSCREEN Gets Rid Of Start Bar.
	// We always want to get a result from this function to check if we failed
	int result = ChangeDisplaySettings(&dmSettings,CDS_FULLSCREEN);	

	// Check if we didn't recieved a good return message From the function
	if(result != DISP_CHANGE_SUCCESSFUL)
	{
		// Display the error message and quit the program
		MessageBox(NULL, "Display Mode Not Compatible", "Error", MB_OK);
		PostQuitMessage(0);
	}
}

///////////////////////////////// CREATE MY WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function creates a window, but doesn't have a message loop
/////
///////////////////////////////// CREATE MY WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

HWND CreateMyWindow(LPSTR strWindowName, int width, int height, DWORD dwStyle, bool bFullScreen, HINSTANCE hInstance)
{
	HWND hWnd;
	WNDCLASS wndclass;
	
	memset(&wndclass, 0, sizeof(WNDCLASS));				// Init the size of the class
	wndclass.style = CS_HREDRAW | CS_VREDRAW;			// Regular drawing capabilities
	wndclass.lpfnWndProc = WinProc;						// Pass our function pointer as the window procedure
	wndclass.hInstance = hInstance;						// Assign our hInstance
	wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);	// General icon
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);		// An arrow for the cursor
	wndclass.hbrBackground = (HBRUSH) (COLOR_WINDOW+1);	// A white window
	wndclass.lpszClassName = "GameTutorials";			// Assign the class name

	RegisterClass(&wndclass);							// Register the class
	
	if(bFullScreen && !dwStyle) 						// Check if we wanted full screen mode
	{													// Set the window properties for full screen mode
		dwStyle = WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
		ChangeToFullScreen();							// Go to full screen
		ShowCursor(FALSE);								// Hide the cursor
	}
	else if(!dwStyle)									// Assign styles to the window depending on the choice
		dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	
	GLWin.g_hInstance = hInstance;							// Assign our global hInstance to the window's hInstance

	RECT rWindow;
	rWindow.left	= 0;								// Set Left Value To 0
	rWindow.right	= width;							// Set Right Value To Requested Width
	rWindow.top	    = 0;								// Set Top Value To 0
	rWindow.bottom	= height;							// Set Bottom Value To Requested Height

	AdjustWindowRect( &rWindow, dwStyle, false);		// Adjust Window To True Requested Size

														// Create the window
	hWnd = CreateWindow("GameTutorials", strWindowName, dwStyle, 0, 0,
						rWindow.right  - rWindow.left, rWindow.bottom - rWindow.top, 
						NULL, NULL, hInstance, NULL);

	if(!hWnd) return NULL;								// If we could get a handle, return NULL

	ShowWindow(hWnd, SW_SHOWNORMAL);					// Show the window
	UpdateWindow(hWnd);									// Draw the window

	SetFocus(hWnd);										// Sets Keyboard Focus To The Window	

	return hWnd;
}

///////////////////////////////// SET UP PIXEL FORMAT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function sets the pixel format for OpenGL.
/////
///////////////////////////////// SET UP PIXEL FORMAT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

bool bSetupPixelFormat(HDC hdc) 
{ 
    PIXELFORMATDESCRIPTOR pfd; 
    int pixelformat; 
 
    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);			// Set the size of the structure
    pfd.nVersion = 1;									// Always set this to 1
														// Pass in the appropriate OpenGL flags
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    pfd.dwLayerMask = PFD_MAIN_PLANE;					// We want the standard mask (this is ignored anyway)
    pfd.iPixelType = PFD_TYPE_RGBA;						// We want RGB and Alpha pixel type
    pfd.cColorBits = SCREEN_DEPTH;						// Here we use our #define for the color bits
    pfd.cDepthBits = SCREEN_DEPTH;						// Depthbits is ignored for RGBA, but we do it anyway
    pfd.cAccumBits = 0;									// No special bitplanes needed
    pfd.cStencilBits = 0;								// We desire no stencil bits
 
	// This gets us a pixel format that best matches the one passed in from the device
    if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == FALSE ) 
    { 
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    } 
 
	// This sets the pixel format that we extracted from above
    if (SetPixelFormat(hdc, pixelformat, &pfd) == FALSE) 
    { 
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    } 
 
    return TRUE;										// Return a success!
}

//////////////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function resizes the viewport for OpenGL.
/////
//////////////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void SizeOpenGLScreen(int width, int height)			// Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero error
	{
		height=1;										// Make the Height Equal One
	}

	glViewport(0,0,width,height);						// Make our viewport the whole window
														// We could make the view smaller inside
														// Our window if we wanted too.
														// The glViewport takes (x, y, width, height)
														// This basically means, what our our drawing boundries

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

														// Calculate The Aspect Ratio Of The Window
														// The parameters are:
														// (view angle, aspect ration of the width to the height, 
														//  The closest distance to the camera before it clips, 
				  // FOV		// Ratio				//  The farthest distance before it stops drawing)
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 1, 500.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}

///////////////////////////////// INITIALIZE GL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function handles all the initialization for OpenGL.
/////
///////////////////////////////// INITIALIZE GL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void InitializeOpenGL(int width, int height) 
{  
    GLWin.g_hDC = GetDC(GLWin.g_hWnd);								// This sets our global HDC
														// We don't free this hdc until the end of our program
    if (!bSetupPixelFormat(GLWin.g_hDC))						// This sets our pixel format/information
        PostQuitMessage (0);							// If there's an error, quit

    GLWin.g_hRC = wglCreateContext(GLWin.g_hDC);					// This creates a rendering context from our hdc
    wglMakeCurrent(GLWin.g_hDC, GLWin.g_hRC);						// This makes the rendering context we just created the one we want to use

	glEnable(GL_TEXTURE_2D);							// Enables Texture Mapping
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing

	SizeOpenGLScreen(width, height);					// Setup the screen translations and viewport
}

///////////////////////////////// DE INIT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function cleans up and then posts a quit message to the window
/////
///////////////////////////////// DE INIT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void DeInit()
{
	if (GLWin.g_hRC)											
	{
		wglMakeCurrent(NULL, NULL);						// This frees our rendering memory and sets everything back to normal
		wglDeleteContext(GLWin.g_hRC);						// Delete our OpenGL Rendering Context	
	}
	
	if (GLWin.g_hDC) 
		ReleaseDC(GLWin.g_hWnd, GLWin.g_hDC);						// Release our HDC from memory
		
	if(GLWin.g_bFullScreen)									// If we were in full screen
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}

	UnregisterClass("GameTutorials", GLWin.g_hInstance);		// Free the window class

	PostQuitMessage (0);								// Post a QUIT message to the window
}


/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// All window handling code was moved to init.cpp. Diagnostic code
// was moved to diagnostics.cpp.
//
// Global variables were moved to main.h and assigned into structures
// to keep the data better organized.
//
// The existing global variables were moved into GTUI_Interface and
// GT3DS_Handler. GTUI_Interface holds all the variables that can be
// manipulated through the UI. GT3DS_Handler holds all the variables
// respective to the Filename of the model, the actual model, textures
// for the model and the CLoad3DS structure.
// 
// Window handling variables are now
// stored in the GLWindow structure in main.h.
//
// The original CVector3, CVector2, tMaterialInfo, t3DObject, and
// t3DModel are all still included in main.h.
//
// FPS code was added so that rendering does not exceed 30 FPS.
//
// The implementation for moving the camera, moving lights, advancing
// frames was moved from the rendering code, so that it would no
// longer be dependent on rendering. Additionally, FPS calculations
// were added to the update functions so that things would happen
// at a constant rate.
//
//
// Tim Graupmann (tgraupmann)
// Game Programmer
// TGraupmann@yahoo.com
// http://tgraupmann.hypermart.net