#include <windows.h>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

#include "animation.h"
#include "matrix.h"


void ApplyRotation(t3DModel *pModel, t3DObject *pObject)
{	
	int rotFrames = pObject->vRotFrameNumber.size();
	if(rotFrames)
	{
		int currentFrame = pModel->currentFrame;
		int currentKey = 1;
		for(int i = 0; i <= currentFrame; i++)
		{
			float rotDegree = 0;
			CVector3 vRotation;	

			if(currentKey < rotFrames)
			{
				int cFrame = pObject->vRotFrameNumber[currentKey];
				int pFrame = pObject->vRotFrameNumber[currentKey - 1];
				rotDegree = pObject->vRotDegree[currentKey] / (cFrame - pFrame);
				vRotation = pObject->vRotation[currentKey];

				if(pObject->vRotFrameNumber[currentKey] <= i)
					currentKey++;
			}
			else
				rotDegree = 0;

			// Rotate the object to the current frame's rotation value
			if(rotDegree)
			{
				// Apply the rotation to our matrix, (pre-processing).
				// The will cause our position to offset, of which we
				// will apply to get back to the position we are supposed
				// to be at.
				glRotatef(rotDegree, vRotation.x, vRotation.y, vRotation.z);
			}
		}
	}	
}

void ApplyTranslation(t3DModel *pModel, t3DObject *pObject, float* T)
{	
	int posFrames = pObject->vPosFrameNumber.size();
	int currentFrame = pModel->currentFrame;

	if(posFrames)
	{
		T[0] = pObject->vPosition[0].x;
		T[1] = pObject->vPosition[0].y;
		T[2] = pObject->vPosition[0].z;		
	}
	else
		T[0] = T[1] = T[2] = 0;

	for(int i=1; i<posFrames; i++)
	{
		if( (pObject->vPosFrameNumber[i] >= currentFrame) || ((i+1)==posFrames) )
		{
			CVector3 vPosition;
			if(pObject->vPosFrameNumber[i] > currentFrame)
			{
				int pFrame = pObject->vPosFrameNumber[i-1];
				int nFrame = pObject->vPosFrameNumber[i];
				int dFrame = nFrame - pFrame;
				CVector3 pPos = pObject->vPosition[i-1] - pObject->vPosition[0];
				CVector3 nPos = pObject->vPosition[i] - pObject->vPosition[0];
				if(currentFrame - pFrame)
				{
					nPos.x *= float(currentFrame - pFrame);
					nPos.y *= float(currentFrame - pFrame);
					nPos.z *= float(currentFrame - pFrame);
				}
				if(nFrame - currentFrame)
				{
					pPos.x *= float(nFrame - currentFrame);
					pPos.y *= float(nFrame - currentFrame);
					pPos.z *= float(nFrame - currentFrame);
				}
				if(dFrame)
				{
					vPosition = pPos + nPos;
					vPosition.x /= float(dFrame);
					vPosition.y /= float(dFrame);
					vPosition.z /= float(dFrame);
				}
			}
			else
				vPosition = pObject->vPosition[i] - pObject->vPosition[0];
			
			glTranslatef(vPosition.x, vPosition.y, vPosition.z);
			T[0] = vPosition.x;
			T[1] = vPosition.y;
			T[2] = vPosition.z;			
			break;
		}
	}
	
	T[3] = (float)sqrt(T[0]*T[0] + T[1]*T[1] + T[2]*T[2]);
}


///////////////////////////////// ANIMATE MODEL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function translates, scales and rotates the object depending on the frame
/////
///////////////////////////////// ANIMATE MODEL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void AnimateModel_Method_1(t3DModel *pModel, t3DObject *pObject)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_1(pModel, pObject->m_ParentNode);	

	int currentFrame = pModel->currentFrame;
	int posFrames = pObject->vPosFrameNumber.size();
	int rotFrames = pObject->vRotFrameNumber.size();
	int i;
	int currentKey = 1;
	int limit;
	t3DObject *parent;	

	parent = pObject;	
	while(parent)
	{		
		if(parent->vPosFrameNumber.size())
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
		}
		parent = parent->m_ParentNode;		
	}
	
	for(i=1; i<posFrames; i++)
	{
		if( (pObject->vPosFrameNumber[i] >= currentFrame) || ((i+1)==posFrames) )
		{
			CVector3 vPosition;
			if(pObject->vPosFrameNumber[i] > currentFrame)
			{
				int pFrame = pObject->vPosFrameNumber[i-1];
				int nFrame = pObject->vPosFrameNumber[i];
				int dFrame = nFrame - pFrame;
				CVector3 pPos = pObject->vPosition[i-1] - pObject->vPosition[0];
				CVector3 nPos = pObject->vPosition[i] - pObject->vPosition[0];
				if(currentFrame - pFrame)
				{
					nPos.x *= float(currentFrame - pFrame);
					nPos.y *= float(currentFrame - pFrame);
					nPos.z *= float(currentFrame - pFrame);
				}
				if(nFrame - currentFrame)
				{
					pPos.x *= float(nFrame - currentFrame);
					pPos.y *= float(nFrame - currentFrame);
					pPos.z *= float(nFrame - currentFrame);
				}
				if(dFrame)
				{
					vPosition = pPos + nPos;
					vPosition.x /= float(dFrame);
					vPosition.y /= float(dFrame);
					vPosition.z /= float(dFrame);
				}
			}
			else
				vPosition = pObject->vPosition[i] - pObject->vPosition[0];

			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
			break;
		}		
	}
	
	parent = pObject;
	limit = -1;	
	while(parent && limit)
	{
		int rotFrames = parent->vRotFrameNumber.size();
		if(rotFrames)
		{
			currentKey = 1;
			for(i = 0; i <= currentFrame; i++)
			{
				float rotDegree = 0;
				CVector3 vRotation;	

				if(currentKey < rotFrames)
				{
					int cFrame = parent->vRotFrameNumber[currentKey];
					int pFrame = parent->vRotFrameNumber[currentKey - 1];
					rotDegree = parent->vRotDegree[currentKey] / (cFrame - pFrame);
					vRotation = parent->vRotation[currentKey];

					if(parent->vRotFrameNumber[currentKey] <= i)
						currentKey++;
				}
				else
					rotDegree = 0;

				// Rotate the object to the current frame's rotation value
				if(rotDegree)
					glRotatef(rotDegree, vRotation.x, vRotation.y, vRotation.z);
			}
		}
		parent = parent->m_ParentNode;
		limit++;
	}	
	
	parent = pObject;	
	while(parent)
	{		
		if(parent->vPosFrameNumber.size())
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				-vPosition.x,
				-vPosition.y,
				-vPosition.z);
		}
		parent = parent->m_ParentNode;
	}	

	parent = pObject;	
	while(parent)
	{
		CVector3 vPivot = parent->vPivot;
		glTranslatef(
			-vPivot.x,
			-vPivot.y,
			-vPivot.z);
		parent = parent->m_ParentNode;		
	}	
}

void AnimateModel_Method_2(t3DModel *pModel, t3DObject *pObject)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_2(pModel, pObject->m_ParentNode);	

	int currentFrame = pModel->currentFrame;
	int posFrames = pObject->vPosFrameNumber.size();
	int rotFrames = pObject->vRotFrameNumber.size();
	int i;
	int currentKey = 1;
	int limit;
	t3DObject *parent;	

	parent = pObject;	
	while(parent)
	{		
		if(parent->vPosFrameNumber.size())
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
		}
		parent = parent->m_ParentNode;		
	}
	
	for(i=1; i<posFrames; i++)
	{
		if( (pObject->vPosFrameNumber[i] >= currentFrame) || ((i+1)==posFrames) )
		{
			CVector3 vPosition;
			if(pObject->vPosFrameNumber[i] > currentFrame)
			{
				int pFrame = pObject->vPosFrameNumber[i-1];
				int nFrame = pObject->vPosFrameNumber[i];
				int dFrame = nFrame - pFrame;
				CVector3 pPos = pObject->vPosition[i-1] - pObject->vPosition[0];
				CVector3 nPos = pObject->vPosition[i] - pObject->vPosition[0];
				if(currentFrame - pFrame)
				{
					nPos.x *= float(currentFrame - pFrame);
					nPos.y *= float(currentFrame - pFrame);
					nPos.z *= float(currentFrame - pFrame);
				}
				if(nFrame - currentFrame)
				{
					pPos.x *= float(nFrame - currentFrame);
					pPos.y *= float(nFrame - currentFrame);
					pPos.z *= float(nFrame - currentFrame);
				}
				if(dFrame)
				{
					vPosition = pPos + nPos;
					vPosition.x /= float(dFrame);
					vPosition.y /= float(dFrame);
					vPosition.z /= float(dFrame);
				}
			}
			else
				vPosition = pObject->vPosition[i] - pObject->vPosition[0];

			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
			break;
		}		
	}
	
	parent = pObject;
	limit = -1;	
	while(parent && limit)
	{
		int rotFrames = parent->vRotFrameNumber.size();
		if(rotFrames)
		{
			currentKey = 1;
			for(i = 0; i <= currentFrame; i++)
			{
				float rotDegree = 0;
				CVector3 vRotation;	

				if(currentKey < rotFrames)
				{
					int cFrame = parent->vRotFrameNumber[currentKey];
					int pFrame = parent->vRotFrameNumber[currentKey - 1];
					rotDegree = parent->vRotDegree[currentKey] / (cFrame - pFrame);
					vRotation = parent->vRotation[currentKey];

					if(parent->vRotFrameNumber[currentKey] <= i)
						currentKey++;
				}
				else
					rotDegree = 0;

				// Rotate the object to the current frame's rotation value
				if(rotDegree)
					glRotatef(rotDegree, vRotation.x, vRotation.y, vRotation.z);
			}
		}
		parent = parent->m_ParentNode;
		limit++;
	}	
	
	parent = pObject;	
	while(parent)
	{		
		if(parent->vPosFrameNumber.size())
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				-vPosition.x,
				-vPosition.y,
				-vPosition.z);
		}
		parent = parent->m_ParentNode;
	}	

	parent = pObject;	
	while(parent)
	{
		CVector3 vPivot = parent->vPivot;
		glTranslatef(
			vPivot.y,
			vPivot.x,
			-vPivot.z);
		parent = parent->m_ParentNode;		
	}	
}

void AnimateModel_Method_3(t3DModel *pModel, t3DObject *pObject)
{
	int currentFrame = pModel->currentFrame;
	int posFrames = pObject->vPosFrameNumber.size();
	int rotFrames = pObject->vRotFrameNumber.size();
	int i;
	int currentKey = 1;
	int limit;
	t3DObject *parent;	
	
	parent = pObject;	
	if(pObject->pVerts)
	{
		CVector3 vVert = parent->pVerts[0];
		glTranslatef(
			vVert.x,
			vVert.y,
			vVert.z);
	}

	for(i=1; i<posFrames; i++)
	{
		if( (pObject->vPosFrameNumber[i] >= currentFrame) || ((i+1)==posFrames) )
		{
			CVector3 vPosition;
			if(pObject->vPosFrameNumber[i] > currentFrame)
			{
				int pFrame = pObject->vPosFrameNumber[i-1];
				int nFrame = pObject->vPosFrameNumber[i];
				int dFrame = nFrame - pFrame;
				CVector3 pPos = pObject->vPosition[i-1] - pObject->vPosition[0];
				CVector3 nPos = pObject->vPosition[i] - pObject->vPosition[0];
				if(currentFrame - pFrame)
				{
					nPos.x *= float(currentFrame - pFrame);
					nPos.y *= float(currentFrame - pFrame);
					nPos.z *= float(currentFrame - pFrame);
				}
				if(nFrame - currentFrame)
				{
					pPos.x *= float(nFrame - currentFrame);
					pPos.y *= float(nFrame - currentFrame);
					pPos.z *= float(nFrame - currentFrame);
				}
				if(dFrame)
				{
					vPosition = pPos + nPos;
					vPosition.x /= float(dFrame);
					vPosition.y /= float(dFrame);
					vPosition.z /= float(dFrame);
				}
			}
			else
				vPosition = pObject->vPosition[i] - pObject->vPosition[0];

			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
			break;
		}		
	}
	
	// Apply rotation keys
	parent = pObject;
	limit = -1;	
	while(parent && limit)
	{
		int rotFrames = parent->vRotFrameNumber.size();
		if(rotFrames)
		{
			currentKey = 1;
			for(i = 0; i <= currentFrame; i++)
			{
				float rotDegree = 0;
				CVector3 vRotation;	

				if(currentKey < rotFrames)
				{
					int cFrame = parent->vRotFrameNumber[currentKey];
					int pFrame = parent->vRotFrameNumber[currentKey - 1];
					rotDegree = parent->vRotDegree[currentKey] / (cFrame - pFrame);
					vRotation = parent->vRotation[currentKey];

					if(parent->vRotFrameNumber[currentKey] <= i)
						currentKey++;
				}
				else
					rotDegree = 0;

				// Rotate the object to the current frame's rotation value
				if(rotDegree)
					glRotatef(rotDegree, vRotation.x, vRotation.y, vRotation.z);
			}
		}
		parent = parent->m_ParentNode;
		limit++;
	}

	// Translate back from pivot location
	parent = pObject;
	while(parent)
	{
		CVector3 vPivot = parent->vPivot;
		glTranslatef(
			-vPivot.x,
			-vPivot.y,
			-vPivot.z);
		parent = parent->m_ParentNode;		
	}

	// Translate to vertex[0] to neutralize vertex values
	parent = pObject;	
	if(pObject->pVerts)
	{
		CVector3 vVert = parent->pVerts[0];
		glTranslatef(
			-vVert.x,
			-vVert.y,
			-vVert.z);
	}	

	if(pObject->m_ParentNode)
		AnimateModel_Method_3(pModel, pObject->m_ParentNode);	
}

void AnimateModel_Method_4(t3DModel *pModel, t3DObject *pObject)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_4(pModel, pObject->m_ParentNode);	

	int currentFrame = pModel->currentFrame;
	int posFrames = pObject->vPosFrameNumber.size();
	int rotFrames = pObject->vRotFrameNumber.size();
	int i;
	int currentKey = 1;
	int limit;
	t3DObject *parent;	
	
	// Translate to only key frame position
	parent = pObject;	
	while(parent)
	{
		if(parent->vPosFrameNumber.size() == 1)
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
		}		
		parent = parent->m_ParentNode;
	}	
	
	for(i=1; i<posFrames; i++)
	{
		if( (pObject->vPosFrameNumber[i] >= currentFrame) || ((i+1)==posFrames) )
		{
			CVector3 vPosition;
			if(pObject->vPosFrameNumber[i] > currentFrame)
			{
				int pFrame = pObject->vPosFrameNumber[i-1];
				int nFrame = pObject->vPosFrameNumber[i];
				int dFrame = nFrame - pFrame;
				CVector3 pPos = pObject->vPosition[i-1];
				CVector3 nPos = pObject->vPosition[i];
				if(currentFrame - pFrame)
				{
					nPos.x *= float(currentFrame - pFrame);
					nPos.y *= float(currentFrame - pFrame);
					nPos.z *= float(currentFrame - pFrame);
				}
				if(nFrame - currentFrame)
				{
					pPos.x *= float(nFrame - currentFrame);
					pPos.y *= float(nFrame - currentFrame);
					pPos.z *= float(nFrame - currentFrame);
				}
				if(dFrame)
				{
					vPosition = pPos + nPos;
					vPosition.x /= float(dFrame);
					vPosition.y /= float(dFrame);
					vPosition.z /= float(dFrame);
				}
			}
			else
				vPosition = pObject->vPosition[i];

			glTranslatef(
				vPosition.x,
				vPosition.y,
				vPosition.z);
			break;
		}		
	}	
	
	// Apply rotation keys
	parent = pObject;
	limit = -1;	
	while(parent && limit)
	{
		int rotFrames = parent->vRotFrameNumber.size();
		if(rotFrames)
		{
			currentKey = 1;
			for(i = 0; i <= currentFrame; i++)
			{
				float rotDegree = 0;
				CVector3 vRotation;	

				if(currentKey < rotFrames)
				{
					int cFrame = parent->vRotFrameNumber[currentKey];
					int pFrame = parent->vRotFrameNumber[currentKey - 1];
					rotDegree = parent->vRotDegree[currentKey] / (cFrame - pFrame);
					vRotation = parent->vRotation[currentKey];

					if(parent->vRotFrameNumber[currentKey] <= i)
						currentKey++;
				}
				else
					rotDegree = 0;

				// Rotate the object to the current frame's rotation value
				if(rotDegree)
					glRotatef(rotDegree, vRotation.x, vRotation.y, vRotation.z);
			}
		}
		parent = parent->m_ParentNode;
		limit++;
	}

	// Translate by minus pivot
	parent = pObject;
	while(parent)
	{
		if(parent->vPosFrameNumber.size())
		{			
			CVector3 vPivot = parent->vPivot;
			glTranslatef(
				-vPivot.x,
				-vPivot.y,
				-vPivot.z);
		}
		parent = parent->m_ParentNode;
	}

	// Translate to back from key frame position 0
	parent = pObject;	
	while(parent)
	{
		//if(parent->vPosFrameNumber.size()) //method 1
		//if(parent->vPosFrameNumber.size() == 1) //method 2
		if(0) // method 3
		{
			CVector3 vPosition = parent->vPosition[0];
			glTranslatef(
				-vPosition.x,
				-vPosition.y,
				-vPosition.z);
		}
		parent = parent->m_ParentNode;
	}
	
	// Translate to vertex[0] to offset object verteces		
	parent = pObject;	
	while(parent)
	{
		//if(parent->vPosFrameNumber.size() > 1) //method 4
		//if(parent->vPosFrameNumber.size()) //method 5
		//if(0) // method 6
		if(1) // method 7
		{
			if(parent->pVerts)
			{
				CVector3 vVert = parent->pVerts[0];
				glTranslatef(
					-vVert.x,
					-vVert.y,
					-vVert.z);
			}
		}
		parent = parent->m_ParentNode;
	}
}

void AnimateModel_Method_5(t3DModel *pModel, t3DObject *pObject)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_5(pModel, pObject->m_ParentNode);

	CMatrix16 CM; // OpenGL's current modelview matrix
	CMatrix16 D; // Tracks matrix deltas
	double P0[4]; //position 0
	double P1[4]; //position 1
	double P2[4]; //position 2
	double DP[4]; //delta position
	float T[4]; //holds the current translation

	// Store the existing matrix
	glPushMatrix();

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Get the current matrix -- could use glPushMatrix
	glGetDoublev(GL_MODELVIEW_MATRIX, CM.M);

	// Measure the initial position
	CM.GetPosition(&P0[0], &P0[1], &P0[2]);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);		
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);

	// Measure the first vertex[0] position
	D.GetPosition(&P1[0], &P1[1], &P1[2]);

	// Reload the current matrix
	//glLoadMatrixd(CM.M);	

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Now we apply the rotation as part of the animation
	ApplyRotation(pModel, pObject);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);	
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);	

	// Measure the second vertex[0] position
	D.GetPosition(&P2[0], &P2[1], &P2[2]);

	/*********DO CALCULATIONS HERE **********/

	// The distance from P0 to P1 should equal vertex 0	
	DP[0] = P1[0] - P0[0];
	DP[1] = P1[1] - P0[1];
	DP[2] = P1[2] - P0[2];
	// of which it is -- excellent

	// The distance from P2 to P1 is the offset that I'm looking for
	DP[0] = P2[0] - P1[0];
	DP[1] = P2[1] - P1[1];
	DP[2] = P2[2] - P1[2];
	// it's there -- good

	// Put the magnitude of each position in the
	// last index.
	P0[3] = sqrt(P0[0]*P0[0] + P0[1]*P0[1] + P0[2]*P0[2]);
	P1[3] = sqrt(P1[0]*P1[0] + P1[1]*P1[1] + P1[2]*P1[2]);
	P2[3] = sqrt(P2[0]*P2[0] + P2[1]*P2[1] + P2[2]*P2[2]);
	DP[3] = sqrt(DP[0]*DP[0] + DP[1]*DP[1] + DP[2]*DP[2]);	

	/////END OF CALCULATIONS/////////////////	

	// Reload the current matrix	
	//glLoadMatrixd(CM.M);

	// Just pop the current matrix
	glPopMatrix();

	// Translate by the difference between the two vertex points
	glTranslated(-DP[0], -DP[1], -DP[2]);

	// Apply the translation keys
	ApplyTranslation(pModel, pObject, T);

	// Apply the rotation keys
	ApplyRotation(pModel, pObject);		

	return;
}

void AnimateModel_Method_6(t3DModel *pModel, t3DObject *pObject, int* Tier)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_6(pModel, pObject->m_ParentNode, Tier);

	(*Tier)++;

	if(*Tier >= 3)
		return;

	CMatrix16 CM; // OpenGL's current modelview matrix
	CMatrix16 D; // Tracks matrix deltas
	double P0[4]; //position 0
	double P1[4]; //position 1
	double P2[4]; //position 2
	double DP[4]; //delta position
	float T[4]; //holds the current translation

	// Store the existing matrix
	glPushMatrix();

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Get the current matrix -- could use glPushMatrix
	glGetDoublev(GL_MODELVIEW_MATRIX, CM.M);

	// Measure the initial position
	CM.GetPosition(&P0[0], &P0[1], &P0[2]);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);		
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);

	// Measure the first vertex[0] position
	D.GetPosition(&P1[0], &P1[1], &P1[2]);

	// Reload the current matrix
	//glLoadMatrixd(CM.M);	

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Now we apply the rotation as part of the animation
	ApplyRotation(pModel, pObject);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);	
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);	

	// Measure the second vertex[0] position
	D.GetPosition(&P2[0], &P2[1], &P2[2]);

	/*********DO CALCULATIONS HERE **********/

	// The distance from P0 to P1 should equal vertex 0	
	DP[0] = P1[0] - P0[0];
	DP[1] = P1[1] - P0[1];
	DP[2] = P1[2] - P0[2];
	// of which it is -- excellent

	// The distance from P2 to P1 is the offset that I'm looking for
	DP[0] = P2[0] - P1[0];
	DP[1] = P2[1] - P1[1];
	DP[2] = P2[2] - P1[2];
	// it's there -- good

	// Put the magnitude of each position in the
	// last index.
	P0[3] = sqrt(P0[0]*P0[0] + P0[1]*P0[1] + P0[2]*P0[2]);
	P1[3] = sqrt(P1[0]*P1[0] + P1[1]*P1[1] + P1[2]*P1[2]);
	P2[3] = sqrt(P2[0]*P2[0] + P2[1]*P2[1] + P2[2]*P2[2]);
	DP[3] = sqrt(DP[0]*DP[0] + DP[1]*DP[1] + DP[2]*DP[2]);	

	/////END OF CALCULATIONS/////////////////	

	// Reload the current matrix	
	//glLoadMatrixd(CM.M);

	// Just pop the current matrix
	glPopMatrix();

	glTranslatef(pObject->vPivot.x, pObject->vPivot.y, pObject->vPivot.z);		

	// Translate by the difference between the two vertex points
	glTranslated(-DP[0], -DP[1], -DP[2]);	

	// Apply the translation keys
	ApplyTranslation(pModel, pObject, T);

	// Apply the rotation keys
	ApplyRotation(pModel, pObject);		

	return;
}

void AnimateModel_Method_7(t3DModel *pModel, t3DObject *pObject, int* Tier)
{
	// Recursively Traverse Head to Toe
	if(pObject->m_ParentNode)
		AnimateModel_Method_7(pModel, pObject->m_ParentNode, Tier);

	(*Tier)++;
	//if(*Tier >= 3)
		//return;

	CMatrix16 CM; // OpenGL's current modelview matrix
	CMatrix16 D; // Tracks matrix deltas
	double P0[4]; //position 0
	double P1[4]; //position 1
	double P2[4]; //position 2
	double DP[4]; //delta position
	float T[4]; //holds the current translation

	// Store the existing matrix
	glPushMatrix();

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Get the current matrix -- could use glPushMatrix
	glGetDoublev(GL_MODELVIEW_MATRIX, CM.M);

	// Measure the initial position
	CM.GetPosition(&P0[0], &P0[1], &P0[2]);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);	
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);

	// Measure the first vertex[0] position
	D.GetPosition(&P1[0], &P1[1], &P1[2]);

	// Reload the current matrix
	//glLoadMatrixd(CM.M);	

	// Use the identity instead of the current matrix
	glLoadIdentity();	

	// Now we apply the rotation as part of the animation
	ApplyRotation(pModel, pObject);	

	// Translate to vertex 0 to measure where it is
	if(pObject->pVerts)
		glTranslatef(pObject->pVerts[0].x, pObject->pVerts[0].y, pObject->pVerts[0].z);	
	
	// Get the current delta'ed matrix
	glGetDoublev(GL_MODELVIEW_MATRIX, D.M);	

	// Measure the second vertex[0] position
	D.GetPosition(&P2[0], &P2[1], &P2[2]);

	// Apply the translation keys
	ApplyTranslation(pModel, pObject, T);

	/*********DO CALCULATIONS HERE **********/

	// The distance from P0 to P1 should equal vertex 0	
	DP[0] = P1[0] - P0[0];
	DP[1] = P1[1] - P0[1];
	DP[2] = P1[2] - P0[2];
	// of which it is -- excellent

	// The distance from P2 to P1 is the offset that I'm looking for
	DP[0] = P2[0] - P1[0];
	DP[1] = P2[1] - P1[1];
	DP[2] = P2[2] - P1[2];
	// it's there -- good

	// Put the magnitude of each position in the
	// last index.
	P0[3] = sqrt(P0[0]*P0[0] + P0[1]*P0[1] + P0[2]*P0[2]);
	P1[3] = sqrt(P1[0]*P1[0] + P1[1]*P1[1] + P1[2]*P1[2]);
	P2[3] = sqrt(P2[0]*P2[0] + P2[1]*P2[1] + P2[2]*P2[2]);
	DP[3] = sqrt(DP[0]*DP[0] + DP[1]*DP[1] + DP[2]*DP[2]);	

	/////END OF CALCULATIONS/////////////////	

	// Reload the current matrix	
	//glLoadMatrixd(CM.M);

	// Just pop the current matrix
	glPopMatrix();	

	// Translate by the difference between the two vertex points
	glTranslated(-DP[0], -DP[1], -DP[2]);	

	// Apply the translation keys
	ApplyTranslation(pModel, pObject, T);

	// Apply the rotation keys
	ApplyRotation(pModel, pObject);

	glTranslatef(-pObject->vPivot.x, -pObject->vPivot.y, -pObject->vPivot.z);

	return;
}