//*******************************************************************//
//                                                                   //
//   - More Advanced Programming Topics -                            //
//                                                                   //
//   $Author:       TGraupmann         tgraupmann@yahoo.com          //
//                                                                   //
//   $Program:	3DS Animation PART 3                                  //
//                                                                   //
//   $Description:	Shows you how to read the .3DS object hierarchy	//
//                                                                   //
//   $Date:	06/01/03                                                 //
//                                                                   //
//*********************************************************************

#include <windows.h>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

#include "ui.h"
#include "materials.h"
#include "diagnostics.h"

///////////////////////////////// PIVOT DIAGNOSTICS \\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This diagnostic function renders extra diagnostic info (pivots, vertex[0])
/////
///////////////////////////////// PIVOT DIAGNOSTICS \\\\\\\\\\\\\\\\\\\\\\\\\\\*
void PivotDiagnostics(t3DModel *pModel, t3DObject *pObject, int CurrentFrame)
{
	// The following diagnostics add spheres at the pivot point of each node,
	// at the vertex[0] point of each node, and at the vertex[X] point of
	// each node. Where X is a global unsigned int that can be manipulated
	// in the UI. If X is larger than the objects number of verteces, then
	// the vertex[last] is used instead.	

	float Specular_Default[4] = {0, 0, 0, 1};
	float Specular[4] = {0.75f, 0.75f, 0.75f, 0.5f};
	float size1 = 0.5f / pModel->MasterScale;
	float size2 = 10.0f / pModel->MasterScale;
	GLUquadric *quad = gluNewQuadric();	

	glPushAttrib(GL_COLOR_BUFFER_BIT);
	glPushAttrib(GL_LIGHTING_BIT);
	glPushAttrib(GL_ENABLE_BIT);

	glDisable(GL_TEXTURE_2D);	
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, Specular_Default);

	// Draw the boundbox
	if(GTUI.DiagMode && 0) // no point to use the (.3DS) bound boxes
	{		
		//glBlendFunc(GTUI.TransparencyTable[GTUI.TransparencySrc],
			//GTUI.TransparencyTable[GTUI.TransparencyDst]); //faster sampling
		glBlendFunc(GTUI.TransparencyTable[1],
			GTUI.TransparencyTable[3]); //faster sampling
		glEnable(GL_BLEND); // Allow transparency

		CVector3 vBoundBoxMin = pObject->vBoundBoxMin;
		CVector3 vBoundBoxMax = pObject->vBoundBoxMax;
		
		if(pObject->vPosition.size())
		{			
			vBoundBoxMin += pObject->vPosition[0] - pObject->vPivot;
			vBoundBoxMax += pObject->vPosition[0] - pObject->vPivot;
		}

		glColor4ub(0, 0, 128, 0);
		
		float mx, my, mz, Mx, My, Mz;
		mx = vBoundBoxMin.x;
		my = vBoundBoxMin.y;
		mz = vBoundBoxMin.z;
		Mx = vBoundBoxMax.x;
		My = vBoundBoxMax.y;
		Mz = vBoundBoxMax.z;
		
		glBegin(GL_QUADS);		
		glVertex3f(mx, My, Mz);
		glVertex3f(mx, My, mz);
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, my, Mz);
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, My, mz);
		glVertex3f(Mx, my, mz);		
		glVertex3f(Mx, my, Mz);		
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, My, mz);		
		glVertex3f(Mx, My, mz);
		glVertex3f(Mx, my, mz);
		glVertex3f(mx, my, Mz);		
		glVertex3f(mx, My, Mz);		
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, my, Mz);
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, my, Mz);		
		glVertex3f(Mx, my, Mz);
		glVertex3f(Mx, my, mz);
		glVertex3f(mx, My, mz);		
		glVertex3f(mx, My, Mz);		
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, My, mz);
		glEnd();

		glColor4ub(255, 0, 0, 255);
		glLineWidth(2.5);
		glBegin(GL_LINES);
		glVertex3f(mx, My, Mz);
		glVertex3f(mx, My, mz);
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, my, Mz);
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, My, mz);
		glVertex3f(Mx, my, mz);		
		glVertex3f(Mx, my, Mz);		
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, My, mz);		
		glVertex3f(Mx, My, mz);
		glVertex3f(Mx, my, mz);
		glVertex3f(mx, my, Mz);		
		glVertex3f(mx, My, Mz);		
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, my, Mz);
		glVertex3f(mx, my, mz);		
		glVertex3f(mx, my, Mz);		
		glVertex3f(Mx, my, Mz);
		glVertex3f(Mx, my, mz);
		glVertex3f(mx, My, mz);		
		glVertex3f(mx, My, Mz);		
		glVertex3f(Mx, My, Mz);
		glVertex3f(Mx, My, mz);
		glEnd();
		glLineWidth(1.0f);

		glDisable(GL_BLEND);
	}	
	
	// PIVOT {0, 0, 0} GREEN sphere
	glColor3ub(0, 255, 0);
	if(pObject->m_NodeID == GTUI.NodeID)
	{		
		if(GTUI.DiagMode)
		{
			gluSphere(quad, size1, 32, 32);
			gluCylinder( quad, 0.0f, size1*10.0f, size2, 16, 16);
		}
	}
	else if(GTUI.DiagMode == 2)
		gluSphere(quad, size1, 32, 32);
	else if(GTUI.DiagMode == 3)
		gluCylinder( quad, 0.0f, size1, size2, 16, 16);

	if(pObject->numOfVerts && GTUI.DiagMode)
	{
		if(pObject->m_NodeID == GTUI.NodeID)
		{
			// Draw a line from pivot to vertex[0]			
			glColor3ub(255, 128, 0);
			glLineWidth(5.0f);
			glBegin(GL_LINES);
			glVertex3f(
				pObject->vPivot.x,
				pObject->vPivot.y,
				pObject->vPivot.z);
			glVertex3f(
				pObject->pVerts[0].x,
				pObject->pVerts[0].y,
				pObject->pVerts[0].z);
			glEnd();
			glLineWidth(1.0f);			
		}		
		
		// OBJECT RED vertex[0]
		glColor3ub(255, 0, 0);
		glPushMatrix();
		if(pObject->pVerts)
		{
			glTranslatef(
				pObject->pVerts[0].x,
				pObject->pVerts[0].y,
				pObject->pVerts[0].z);
		}
		if(pObject->m_NodeID == GTUI.NodeID)
		{			
			if(GTUI.DiagMode)
			{
				gluCylinder( quad, 0.0f, size1, size2*2.0f, 16, 16);
				gluSphere(quad, size1, 32, 32);
			}
		}
		else if(GTUI.DiagMode == 2)
			gluSphere(quad, size1, 32, 32);
		else if(GTUI.DiagMode == 3)
			gluCylinder( quad, 0.0f, size1, size2*2.0f, 16, 16);
		glPopMatrix();

		// Error handling
		if(GTUI.VertexID >= pObject->numOfVerts)
			return;
		
		// OBJECT BLUE vertex[X]
		glColor3ub(0, 0, 255);
		glPushMatrix();
		glTranslatef(
			pObject->pVerts[GTUI.VertexID].x,
			pObject->pVerts[GTUI.VertexID].y,
			pObject->pVerts[GTUI.VertexID].z);
		if(pObject->m_NodeID == GTUI.NodeID)
		{			
			if(GTUI.DiagMode)
			{
				gluSphere(quad, size1, 32, 32);
				gluCylinder( quad, 0.0f, size1, size2, 16, 16);
			}
		}
		else if(GTUI.DiagMode == 2)
			gluSphere(quad, size1, 32, 32);
		else if(GTUI.DiagMode == 3)
			gluCylinder( quad, 0.0f, size1, size2, 16, 16);
		glPopMatrix();


		// MAKE ANIMATION POSITION[0] YELLOW
		glColor3ub(255, 255, 0);
		glPushMatrix();
		if(pObject->vPosition.size() > 1)
		{
			glTranslatef(
				pObject->vPosition[1].x - pObject->vPosition[0].x,
				pObject->vPosition[1].y - pObject->vPosition[0].y,
				pObject->vPosition[1].z - pObject->vPosition[0].z);
		}
		if(pObject->m_NodeID == GTUI.NodeID)
		{			
			if(GTUI.DiagMode)
			{
				gluCylinder( quad, 0.0f, size1, size2*1.5f, 16, 16);
				gluSphere(quad, size1, 32, 32);
			}
		}
		else if(GTUI.DiagMode == 2)
			gluSphere(quad, size1, 32, 32);
		else if(GTUI.DiagMode == 3)
		{
			gluSphere(quad, size1, 32, 32);
			gluCylinder( quad, 0.0f, size1, size2*1.5f, 16, 16);
		}
		glPopMatrix();
	}


	// MAKE ANIMATION PIVOT MAGENTA
	glColor3ub(255, 0, 255);
	glPushMatrix();
	
	glTranslatef(
		pObject->vPivot.x,
		pObject->vPivot.y,
		pObject->vPivot.z);
	
	if(pObject->m_NodeID == GTUI.NodeID)
	{			
		if(GTUI.DiagMode)
		{
			gluCylinder( quad, 0.0f, size1*1.5f, size2*2.5f, 16, 16);
			gluSphere(quad, size1, 32, 32);
		}
	}
	else if(GTUI.DiagMode == 2)
		gluSphere(quad, size1, 32, 32);
	else if(GTUI.DiagMode == 3)
		gluCylinder( quad, 0.0f, size1, size2*1.5f, 16, 16);
	glPopMatrix();	

	gluDeleteQuadric(quad);
	glPopAttrib();
}


///////////////////////////////// IS PARENT QUERY \\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This diagnostic function returns whether a node is in the hierarchy
/////
///////////////////////////////// IS PARENT QUERY \\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

bool IsParentQuery(t3DObject *pObject, unsigned short NodeID)
{
   // This function provides additional diagnostics. The current node
	// will be highlighted if diagnostics are activated. Any nodes that
	// are children of the current node will also be highlighted.
	if(pObject->m_NodeID == NodeID)
      return 1;

	if(NodeID == 65535)
		if(!pObject->m_ParentNode)
			return 1;

	if(pObject->m_ParentNode)
		return IsParentQuery(pObject->m_ParentNode, NodeID);
	return 0;
}

/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// This file is entirely implementation for the diagnostics that can be  
// manipulated through the UI. Functions show pivot points and diagnostic
// lights as spheres.
// 
// 
// Tim Graupmann (tgraupmann)
// Game Programmer
// TGraupmann@yahoo.com
// http://tgraupmann.hypermart.net
