#ifndef _MAIN_H
#define _MAIN_H

#include <windows.h>
#include <gl\gl.h> // Header File For The OpenGL32 Library
#include <gl\glaux.h> // Header File for OpenGL aux functions
#include <crtdbg.h>

#define SCREEN_WIDTH 800	// We want our screen width 800 pixels
#define SCREEN_HEIGHT 600	// We want our screen height 600 pixels
#define SCREEN_DEPTH 16		// We want 16 bits per pixel


// This file includes all of the model structures that are needed to load
// in a .3DS file.  If you intend to do animation you will need to add on
// to this.  These structures only support the information that is needed
// to load the objects in the scene and their associative materials.

// This creates a texture and stores it in the texture array with it's ID.
void CreateTexture(UINT textureArray[], LPSTR strFileName, int textureID);

// This draws everything to the screen
void RenderScene();

/////////////////////////////////////////////////////////////////////////////////
///////////////////////////////WIN32 WINDOW HANDLING\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Code to follow is explicitly for Win32 Window Handling
/////
///////////////////////////////WIN32 WINDOW HANDLING\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////////////////////////////////////////////////////////////////////////////////

// Keep window information grouped together
typedef struct {
	bool  g_bFullScreen;				// Set full screen as default
	HWND  g_hWnd;						// This is the handle for the window
	RECT  g_rRect;						// This holds the window dimensions
	HDC   g_hDC;						// General HDC - (handle to device context)
	HGLRC g_hRC;						// General OpenGL_DC - Our Rendering Context for OpenGL
	HINSTANCE g_hInstance;			// This holds our window hInstance
} GLWindow;

// Make the global window GLWin variables external and accessable
extern GLWindow GLWin;

// This is our MAIN() for windows
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hprev, PSTR cmdline, int ishow);

// The window proc which handles all of window's messages.
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

// This controls our main program loop
WPARAM MainLoop();

// This changes the screen to full screen mode
void ChangeToFullScreen();

// This is our own function that makes creating a window modular and easy
HWND CreateMyWindow(LPSTR strWindowName, int width, int height, DWORD dwStyle, bool bFullScreen, HINSTANCE hInstance);

// This allows us to configure our window for OpenGL and backbuffered
bool bSetupPixelFormat(HDC hdc);

// This inits our screen translations and projections
void SizeOpenGLScreen(int width, int height);

// This sets up OpenGL
void InitializeOpenGL(int width, int height);

// This initializes the whole program
void Init(HWND hWnd);

// This frees all our memory in our program
void DeInit();


#endif 


/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// Not only did We add new variables to the t3DModel and t3DObject structs, we
// evolved CVector3, CVector2, tFace, tMaterialInfo, t3DObject, and t3DModel
// into full-fledged classes with constructors, copy constructors, destructors,
// and some operator overloads. This was all done to make sure that no memory
// corruption is going on.
// To the model we added some variables to store the current frame and number of
// frames in the animation.  To the object structure we added 4 vectors to store
// the postion, rotation axis and scale for each frame, as well as the rotation
// degree for that axis.  We also added the amount of position, rotation and scale
// frames.  These aren't used much, except when filling in the STL vectors.
// 
// Look at 3ds.cpp or Main.cpp for a better idea of how these are used and how
// the animation is read and utilized.
// 
//
//
// Ben Humphrey (DigiBen)
// Game Programmer
// DigiBen@GameTutorials.com
// Co-Web Host of www.GameTutorials.com
//
//
