#ifndef _MATRIX_H
#define _MATRIX_H


#include <math.h>


class CMatrix16
{
public:
	double M[16];

	CMatrix16()
	{
		for(short i=0; i<16; i++)
			M[i] = !(i%5);
	}	

	CMatrix16(const CMatrix16& copy)
	{
		for(short i=0; i<16; i++)
			M[i] = copy.M[i];
	}

	~CMatrix16()
	{
		for(short i=0; i<16; i++)
			M[i] = !(i%5);
	}

	const CMatrix16 &operator=(const CMatrix16& copy)
	{
		for(short i=0; i<16; i++)
			M[i] = copy.M[i];

		return copy;
	}

	const CMatrix16 operator+(const CMatrix16& copy)
	{
		CMatrix16 R;
		for(short i=0; i<16; i++)
			R.M[i] = M[i] + copy.M[i];

		return R;
	}

	/*
	friend const CMatrix16 operator+(const CMatrix16 m1, const CMatrix16 m2)
	{
		CMatrix16 R;
		for(short i=0; i<16; i++)
			R.M[i] = m1.M[i] + m2.M[i];		

		return R;
	}
	*/

	const CMatrix16 operator*(const double k)
	{
		CMatrix16 R;
		for(short i=0; i<16; i++)
			R.M[i] = M[i] * k;

		return R;
	}

	friend const CMatrix16 operator*(const double k, const CMatrix16 m)
	{
		CMatrix16 R;
		for(short i=0; i<16; i++)
			R.M[i] = m.M[i] * k;

		return R;
	}

	const CMatrix16 operator*(const CMatrix16 m)
	{
		CMatrix16 R;
		
		// [ A ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |  <->
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//   <|>		
		R.M[0]  = M[0]*m.M[0] + M[4]*m.M[1] + M[8]*m.M[2] + M[12]*m.M[3];

		// [ B ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |  <->
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//        <|>		
		R.M[4]  = M[0]*m.M[4] + M[4]*m.M[5] + M[8]*m.M[6] + M[12]*m.M[7];

		// [ C ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |  <->
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//             <|>		
		R.M[8]  = M[0]*m.M[8] + M[4]*m.M[9] + M[8]*m.M[10] + M[12]*m.M[11];

		// [ D ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |  <->
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//                  <|>		
		R.M[12]  = M[0]*m.M[12] + M[4]*m.M[13] + M[8]*m.M[14] + M[12]*m.M[15];


		// [ E ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |  <->
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//   <|>		
		R.M[1]  = M[1]*m.M[0] + M[5]*m.M[1] + M[9]*m.M[2] + M[13]*m.M[3];

		// [ F ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |  <->
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//        <|>		
		R.M[5]  = M[1]*m.M[4] + M[5]*m.M[5] + M[9]*m.M[6] + M[13]*m.M[7];

		// [ G ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |  <->
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//             <|>		
		R.M[9]  = M[1]*m.M[8] + M[5]*m.M[9] + M[9]*m.M[10] + M[13]*m.M[11];

		// [ H ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |  <->
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//                  <|>		
		R.M[13]  = M[1]*m.M[12] + M[5]*m.M[13] + M[9]*m.M[14] + M[13]*m.M[15];


		// [ I ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |  <->
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//   <|>		
		R.M[2]  = M[2]*m.M[0] + M[6]*m.M[1] + M[10]*m.M[2] + M[14]*m.M[3];

		// [ J ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |  <->
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//        <|>		
		R.M[6]  = M[2]*m.M[4] + M[6]*m.M[5] + M[10]*m.M[6] + M[14]*m.M[7];

		// [ K ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |  <->
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//             <|>		
		R.M[10]  = M[2]*m.M[8] + M[6]*m.M[9] + M[10]*m.M[10] + M[14]*m.M[11];

		// [ L ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |  <->
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//                  <|>		
		R.M[14]  = M[2]*m.M[12] + M[6]*m.M[13] + M[10]*m.M[14] + M[14]*m.M[15];


		// [ M ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |  <->
		// ----------------------
		//   <|>		
		R.M[3]  = M[3]*m.M[0] + M[7]*m.M[1] + M[11]*m.M[2] + M[15]*m.M[3];

		// [ N ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |  <->
		// ----------------------
		//        <|>		
		R.M[7]  = M[3]*m.M[4] + M[7]*m.M[5] + M[11]*m.M[6] + M[15]*m.M[7];

		// [ O ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |  <->
		// ----------------------
		//             <|>		
		R.M[11]  = M[3]*m.M[8] + M[7]*m.M[9] + M[11]*m.M[10] + M[15]*m.M[11];

		// [ P ] : Multiply this row and column 
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |  <->
		// ----------------------
		//                  <|>		
		R.M[15]  = M[3]*m.M[12] + M[7]*m.M[13] + M[11]*m.M[14] + M[15]*m.M[15];

		return R;
	}

	void SetZero(void)
	{
		for(short i=0; i<16; i++)
			M[i] = 0;
	}

	void SetIdentity(void)
	{
		for(short i=0; i<16; i++)
			M[i] = !(i%5);
	}

	CMatrix16 Scale(double x, double y, double z)
	{
		// glScale multiplies the current matrix
		// with the following matrix.
		//
		// ----------------------
		// |  x ,  0 ,  0 ,  0  |
		// ----------------------
		// |  0 ,  y ,  0 ,  0  |
		// ----------------------
		// |  0 ,  0 ,  z ,  0  |
		// ----------------------
		// |  0 ,  0 ,  0 ,  1  |
		// ----------------------

		// Our matrix will do the same:		
		//
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//
		// a00 will be set to x
		// f05 will be set to y
		// k10 will be set to z
		//
		// and then the current
		// matrix will be multiplied
		// by this new transform

		CMatrix16 S;
		S.SetIdentity();		
		S.M[0] = x;
		S.M[5] = y;
		S.M[10] = z;

		*this = *this * S;

		return *this;
	}

	CMatrix16 Translate(double x, double y, double z)
	{
		// glTranslate multiplies the current matrix
		// with the following matrix.
		//
		// ----------------------
		// |  1 ,  0 ,  0 ,  x  |
		// ----------------------
		// |  0 ,  1 ,  0 ,  y  |
		// ----------------------
		// |  0 ,  0 ,  1 ,  z  |
		// ----------------------
		// |  0 ,  0 ,  0 ,  1  |
		// ----------------------

		// Our matrix will do the same:		
		//
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//
		// d12 will be set to x
		// h13 will be set to y
		// i14 will be set to z
		//
		// and then the current
		// matrix will be multiplied
		// by this new transform

		CMatrix16 T;
		T.SetIdentity();		
		T.M[12] = x;
		T.M[13] = y;
		T.M[14] = z;

		*this = *this * T;

		return *this;
	}

	CMatrix16 Rotate(double rotDegree, double X, double Y, double Z)
	{
		// glRotate multiplies the current matrix
		// with the following matrix.
		//
		// ----------------------
		// |  xx(1-c)+c ,  xy(1-c)-zs ,  xz(1-c)+ys ,  0  |
		// ----------------------
		// |  yx(1-c)+zs ,  yy(1-c)+c ,  yz(1-c)-xs ,  0  |
		// ----------------------
		// |  xz(1-c)-ys ,  yz(1-c)+xs ,  zz(1-c)+c ,  0  |
		// ----------------------
		// |  0 ,  0 ,  0 ,  1  |
		// ----------------------

		// Our matrix will do the same:
		//
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------
		//
		// a00 will be set to xx(1-c)+c
		// b04 will be set to xy(1-c)-zs
		// c08 will be set to xz(1-c)+ys
		//
		// e01 will be set to yx(1-c)+zs
		// f05 will be set to yy(1-c)+c
		// g09 will be set to yz(1-c)-xs
		//
		// i02 will be set to xz(1-c)-ys
		// j06 will be set to yz(1-c)+xs
		// k10 will be set to zz(1-c)+c
		//
		// and then the current
		// matrix will be multiplied
		// by this new transform

		//normalize angle, x, y, z
		double magnitude = sqrt(X*X + Y*Y + Z*Z);
		double x=X, y=Y, z=Z;
		if(magnitude != 1.0)
		{
			x /= magnitude;
			y /= magnitude;
			z /= magnitude;
		}
		
		//covert angle to radians
		double angle = (rotDegree / 180.0) * 3.1415926535897932384626433832795;
		double c = (double)cos(angle);
		double s = (double)sin(angle);
		CMatrix16 R;
		R.SetIdentity();		
		R.M[0] = x*x*(1-c)+c;		
		R.M[4] = x*y*(1-c)-z*s;		
		R.M[8] = x*z*(1-c)+y*s;		
		R.M[1] = y*x*(1-c)+z*s;		
		R.M[5] = y*y*(1-c)+c;		
		R.M[9] = y*z*(1-c)-x*s;		
		R.M[2] = x*z*(1-c)-y*s;		
		R.M[6] = y*z*(1-c)+x*s;		
		R.M[10] = z*z*(1-c)+c;

		*this = *this * R;

		return *this;
	}

	void GetPosition(double* X, double* Y, double* Z)
	{
		// Here is our current matrix:
		//
		// ----------------------
		// | a00, b04, c08, d12 |
		// ----------------------
		// | e01, f05, g09, h13 |
		// ----------------------
		// | i02, j06, k10, l14 |
		// ----------------------
		// | m03, n07, o11, p15 |
		// ----------------------		

		*X = M[12];
		*Y = M[13];
		*Z = M[14];		
	}
};


#endif