#ifndef _MODELS_H
#define _MODELS_H

#include <windows.h>
#include <vector>
using namespace std;

#include "vectorxf.h"
#include "objects.h"
#include "materials.h"


// This holds our model information.  This should also turn into a robust class.
// We use STL's (Standard Template Library) vector class to ease our link list burdens. :)
class t3DModel
{
public:
	int numOfObjects;			// The number of objects in the model
	int numOfMaterials;			// The number of materials for the model

	int numberOfFrames;			// The number of frames of animation this model (at least 1)
	int currentFrame;			// The current frame of animation that the model is in

	vector<tMaterialInfo> pMaterials;	// The list of material information (Textures and colors)
	vector<t3DObject> pObject;			// The object list for our model	

	// The master scale value that should be applied to all
	// the scale track values for each frame of the animation.	
	float MasterScale;

	// This is a hash table that allows nodes to be referred by node ids
	// [ NodeID ] == Object Index
	vector<unsigned short> m_NodeHashTable;

	// Keeps track of the time each model update occurs
	long modClock;

	// Counts the number of model updates for each second
	short MODCount;

	// Tracks the number of model updates per second
	short ModelFPS;

	// Holds the path and file name of the (.3DS) model
	char m_FileName[255];

	// This function shows the given model using display lists
	// or not, depending on the UI variables that are set.	
	void ShowModel(void);

	// This function sets the material/textures for the object.
	void SetMaterial(t3DObject *pObject);

	// This function draws the object and uses all the drawing
	// calls whether in a display list or not.
	void DrawObject(t3DObject *pObject);

	// This function creates a display list using displayListOffset as
	// the index to be used. The function then calls DrawObject and
	// compiles all the drawing commands therein.
	void CreateDisplayList(t3DObject *pObject, int displayListOffset);
	
	t3DModel::t3DModel();	// Constructor
	t3DModel::t3DModel(const t3DModel& copy); // Copy constructor
	t3DModel::~t3DModel(); // Destructor	
};

#endif