#include <windows.h>

#include "objects.h"


// This is our face structure.  This is is used for indexing into the vertex 
// and texture coordinate arrays.  From this information we know which vertices
// from our vertex array go to which face, along with the correct texture coordinates.
tFace::tFace()
{
	memset(vertIndex, 0, sizeof(int) * 3);
	memset(coordIndex, 0, sizeof(int) * 3);
}
tFace::tFace(const tFace& copy)
{
	memcpy(vertIndex, copy.vertIndex, sizeof(int) * 3);
	memcpy(coordIndex, copy.coordIndex, sizeof(int) * 3);
}
tFace::~tFace()
{
	memset(vertIndex, 0, sizeof(int) * 3);
	memset(coordIndex, 0, sizeof(int) * 3);
}	


// This holds all the information for our model/scene. 
// You should eventually turn into a robust class that 
// has loading/drawing/querying functions like:
// LoadModel(...); DrawObject(...); DrawModel(...); DestroyModel(...);

// Constructor
t3DObject::t3DObject()
{
	pIndices = 0;
	numOfVerts = 0;
	numOfFaces = 0;
	numTexVertex = 0;
	materialID = -1;
	bHasTexture = false;
	memset(strName, 0, sizeof(char) * 255);
	scaleFrames = 0;		
	memset(&vPivot, 0, sizeof(CVector3));
	memset(&vBoundBoxMin, 0, sizeof(CVector3));
	memset(&vBoundBoxMax, 0, sizeof(CVector3));		
	vPosFrameNumber.clear();
	vPosition.clear();
	vRotFrameNumber.clear();
	vRotation.clear();
	vScale.clear();
	vRotDegree.clear();
	pIndices = 0;
	pVerts = 0;
	pNormals = 0;
	pTexVerts = 0;
	pFaces = 0;
	m_NodeID = 65535;
	m_ParentNode = NULL;
	m_GroupToggle = false;
	m_LinkedObjectToggle = false;
	m_DisplayListID = 0;
}

// Copy constructor
t3DObject::t3DObject(const t3DObject& copy)
{
	SetObject(&copy); // helper fuction does the copy
}

// Destructor
t3DObject::~t3DObject()
{
	ClearObject();					// prevents memory leak
}

// Equals operator	
const t3DObject& t3DObject::operator=(const t3DObject &right)
{
	if ( &right != this ) {		// avoid self assignment
		ClearObject();				// prevents memory leak
		SetObject(&right);		// helper fuction does the copy
	}
	return *this;
}

void t3DObject::ClearObject(void)
{		
	if(pIndices)
	{
		delete [numOfFaces*3] pIndices;			
		pIndices = 0;
	}
	if(pVerts)
	{
		delete [numOfVerts] pVerts;			
		pVerts = 0;
	}
	if(pNormals)
	{
		delete [numOfVerts] pNormals;
		pNormals = 0;
	}
	if(pTexVerts)
	{
		delete [numTexVertex] pTexVerts;
		pTexVerts = 0;
	}
	if(pFaces)
	{
		delete [numOfFaces] pFaces;
		pFaces = 0;
	}
	pIndices = 0;
	numOfVerts = 0;
	numOfFaces = 0;
	numTexVertex = 0;
	materialID = -1;
	bHasTexture = false;
	memset(strName, 0, sizeof(char) * 255);
	scaleFrames = 0;
	memset(&vPivot, 0, sizeof(CVector3));
	memset(&vBoundBoxMin, 0, sizeof(CVector3));
	memset(&vBoundBoxMax, 0, sizeof(CVector3));		
	vPosFrameNumber.clear();
	vPosition.clear();
	vRotFrameNumber.clear();
	vRotation.clear();
	vScale.clear();
	vRotDegree.clear();
	m_NodeID = 65535;
	m_ParentNode = NULL;
	m_GroupToggle = false;
	m_LinkedObjectToggle = false;
	m_DisplayListID = 0;
}

void t3DObject::SetObject(const t3DObject *Object)
{		
	if(Object->pIndices)
	{
		pIndices = new UINT[ Object->numOfFaces * 3 ];
		memcpy(pIndices, Object->pIndices, sizeof(UINT) * Object->numOfFaces * 3);			
	}
	else
		pIndices = 0;
	if(Object->pVerts)
	{
		pVerts = new CVector3[ Object->numOfVerts ];
		memcpy(pVerts, Object->pVerts, sizeof(CVector3) * Object->numOfVerts);
	}
	else
		pVerts = NULL;
	if(Object->pNormals)
	{
		pNormals = new CVector3[ Object->numOfVerts ];
		memcpy(pNormals, Object->pNormals, sizeof(CVector3) * Object->numOfVerts);
	}
	else
		pNormals = NULL;
	if(Object->pTexVerts)
	{
		pTexVerts = new CVector2[ Object->numTexVertex ];
		memcpy(pTexVerts, Object->pTexVerts, sizeof(CVector2) * Object->numTexVertex);			
	}
	else
		pTexVerts = NULL;
	if(Object->pFaces)
	{
		pFaces = new tFace[ Object->numOfFaces ];
		memcpy(pFaces, Object->pFaces, sizeof(tFace) * Object->numOfFaces);			
	}
	else
		pFaces = NULL;
	numOfVerts = Object->numOfVerts;
	numOfFaces = Object->numOfFaces;
	numTexVertex = Object->numTexVertex;
	materialID = Object->materialID;
	bHasTexture = Object->bHasTexture;
	memcpy(strName, Object->strName, sizeof(char) * 255);
	scaleFrames = Object->scaleFrames;
	vPivot = Object->vPivot;
	vBoundBoxMin = Object->vBoundBoxMin;
	vBoundBoxMax = Object->vBoundBoxMax;
	
	// Copy vector lists

	for(unsigned short i = 0; i < Object->vPosFrameNumber.size(); i++)
		vPosFrameNumber.push_back( Object->vPosFrameNumber[i] );		

	for(i = 0; i < Object->vPosition.size(); i++)
		vPosition.push_back( Object->vPosition[i] );

	for(i = 0; i < Object->vRotFrameNumber.size(); i++)
		vRotFrameNumber.push_back( Object->vRotFrameNumber[i] );

	for(i = 0; i < Object->vRotation.size(); i++)
		vRotation.push_back( Object->vRotation[i] );

	for(i = 0; i < Object->vScale.size(); i++)
		vScale.push_back( Object->vScale[i] );

	for(i = 0; i < Object->vRotDegree.size(); i++)
		vRotDegree.push_back( Object->vRotDegree[i] );

	m_NodeID = Object->m_NodeID;
	m_ParentNode = Object->m_ParentNode;
	m_GroupToggle = Object->m_GroupToggle;
	m_LinkedObjectToggle = Object->m_LinkedObjectToggle;
	m_DisplayListID = Object->m_DisplayListID;
}