#ifndef _OBJECTS_H
#define _OBJECTS_H

#include <windows.h>
#include <vector>
using namespace std;

#include "vectorxf.h"

// This is our face structure.  This is is used for indexing into the vertex 
// and texture coordinate arrays.  From this information we know which vertices
// from our vertex array go to which face, along with the correct texture coordinates.
class tFace
{
public:
	int vertIndex[3];	// indicies for the verts that make up this triangle
	int coordIndex[3];	// indicies for the tex coords to texture this face

	tFace();					// Constructor	
	tFace(const tFace& copy);	// Copy constructor	
	~tFace();					// Destructor
};

// This holds all the information for our model/scene. 
// You should eventually turn into a robust class that 
// has loading/drawing/querying functions like:
// LoadModel(...); DrawObject(...); DrawModel(...); DestroyModel(...);
class t3DObject
{
public:
	int  numOfVerts;			// The number of verts in the model
	int  numOfFaces;			// The number of faces in the model
	int  numTexVertex;			// The number of texture coordinates
	int  materialID;			// The texture ID to use, which is the index into our texture array
	bool bHasTexture;			// This is TRUE if there is a texture map for this object
	char strName[255];			// The name of the object

	int scaleFrames;			// The number of key frames for the scaling

	CVector3 vPivot;			// The object pivot point (The local axis it rotates around)

	CVector3 vBoundBoxMin;		// Object min bound box in world coordinates
	CVector3 vBoundBoxMax;		// Object max bound box in world coordinates

	vector<unsigned short> vPosFrameNumber; //The frame numbers for each keyframe position
	vector<CVector3> vPosition;	// The object's current position list

	vector<unsigned short> vRotFrameNumber; //The frame numbers for each keyframe rotation
	vector<CVector3> vRotation;	// The object's current rotation list
	vector<CVector3> vScale;	// The object's current scale list

	vector<float> vRotDegree;	// The object's current rotation degree (associated with vRotation)

/////// * /////////// * /////////// * NEW * /////// * /////////// * /////////// *
	UINT      *pIndices;		// The straight face indices in a row, used for vertex arrays
/////// * /////////// * /////////// * NEW * /////// * /////////// * /////////// *

	CVector3  *pVerts;			// The object's vertices
	CVector3  *pNormals;		// The object's normals
	CVector2  *pTexVerts;		// The texture's UV coordinates
	tFace *pFaces;				// The faces information of the object

	// These are the variables that make the object hierarchy possible
	unsigned short m_NodeID;	// The primary key for each animation node
	t3DObject* m_ParentNode;	// Points to the parent node of the object
	bool m_GroupToggle;			// This indicates whether the object is a group
	bool m_LinkedObjectToggle; // Indicates a linked object hierachy

	int m_DisplayListID; // Holds the display list for each object

	t3DObject(); // Constructor		
	t3DObject(const t3DObject& copy); // Copy constructor
	~t3DObject(); // Destructor
	const t3DObject &operator=(const t3DObject &right); // Equals operator
	void ClearObject(void); // Helper that clears the object
	void SetObject(const t3DObject *Object); // Helper that sets the object
};

#endif