#ifndef _UI_H
#define _UI_H

#include "camera.h"
#include "3ds.h"


#define MAX_TEXTURES 100 // The maximum amount of textures to load

// The following are structures created to hold all the
// UI manipulated variables, and it keeps things cleaner.

// Keep all UI variables together in a structure
typedef struct {
	float CameraPosition[3]; // Holds the camera position
	int ViewMode;              // We want the default drawing mode to be normal
	bool LightingToggle;       // Flag Toogles Whether lighting Is Used
	float LightPosition0[3]; // Holds the X, Y, Z position of Light0
	CCamera	Camera; // This is our global camera object
	unsigned short CameraMode; // Alters common view angles
	unsigned short DiagMode;	// Toggles pivot diagnostics
	unsigned short PivotDiag1; // Objects that lack parents
	unsigned short PivotDiag2; // Parents that have parents
	unsigned short PivotDiag3; // Parents that lack parents
	unsigned short TransID;		// Indicates selected key position
	unsigned short VertexID;	// Indicates selected vertex
	unsigned short NodeID;		// Indicates selected object
	bool FrameDelayToggle;		// Change speed of animation
	float Black4f[4];
	int TransparencyTable[8]; // Transparency Table
	int TransparencySrc;			// Transparency source table
	int TransparencyDst;			// Transparency destination table
	long camClock;		// Static clock for camera updates
	long rdrClock;		// Static clock for renderings
	short RDRCount;	// Counts the number of displays in a second
	short CAMCount;	// Counts the number of camera updates per second	
	float RenderFPS;	// Tracks the number of displays per second
	short CameraFPS;	// Tracks the number of camera updates per second
	int ModelCount;	// Counts the number of models that have been imported
	int SelectedModel; // The current model being displayed
	bool ModelMode; // Toggles mult/single model mode
	int TotalNormals; // Count the number of normals
	int TotalTexVerts; // Count the number of tex verts
	int TotalVerts; // Count the number of verts
	int TotalDisplayLists; // Count the number of display lists
	bool SignalDLRecompile; // Indicates that the display lists should be recompiled
	bool UseDLToggle; // Toggle the use of display lists
	bool UseAttrib; // Toggle pushing/popping attributes
	bool UseMaterials; // Toggle using materials
	int g_pivotType; // Toggle pivot animation types
	CLoad3DS cLoad3ds; // This is 3DS class.  This should go in a good model class.    
    t3DModel c3DModel[100]; // This holds the 3D Model info that we load in
    unsigned int Texture[MAX_TEXTURES]; // This holds the texture info, referenced by an ID
} GTUI_Interface;

// Keep all UI variables together in a structure
extern GTUI_Interface GTUI;

// Initialize UI Variables
void initGTUI(void);

#endif