#ifndef _VECTORXF_H
#define _VECTORXF_H

#include <windows.h>
#include <math.h>

// This is our 3D point class.  This will be used to store the vertices of our model.
class CVector3 
{
	
public:
	float x, y, z;
	
	CVector3()
	{		
		x = 0;
		y = 0;
		z = 0;		
	}

	CVector3(const CVector3& copy): x(copy.x), y(copy.y), z(copy.z)
	{ // empty	
	}
	
	CVector3(float X, float Y, float Z) : x(X), y(Y), z(Z)
	{ // empty		
	}	

	~CVector3()
	{
		x = 0;
		y = 0;
		z = 0;		
	}

	// Operator overloads	
		
	const CVector3 &operator=(const CVector3& rhs)
	{
		x = rhs.x;
		y = rhs.y;
		z = rhs.z;
		return *this;
	}

	const CVector3 &operator+=(const CVector3& rhs)
	{
		x += rhs.x;
		y += rhs.y;
		z += rhs.z;
		return *this;
	}

	const CVector3 &operator-=(const CVector3& rhs)
	{
		x -= rhs.x;
		y -= rhs.y;
		z -= rhs.z;
		return *this;
	}

	CVector3 operator+(const CVector3& rhs)
	{
		CVector3 temp;
		memset(&temp, 0, sizeof(CVector3));
		temp.x = x + rhs.x;
		temp.y = y + rhs.y;
		temp.z = z + rhs.z;
		return temp;
	}

	CVector3 operator-(const CVector3& rhs)
	{
		CVector3 temp;
		memset(&temp, 0, sizeof(CVector3));
		temp.x = x - rhs.x;
		temp.y = y - rhs.y;
		temp.z = z - rhs.z;
		return temp;
	}
};

/////////////////////////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This returns the magnitude of a vector
/////
/////////////////////////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

float Magnitude(CVector3 vNormal);

/////////////////////////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This returns the magnitude between two vectors
/////
/////////////////////////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
float Magnitude(CVector3 VPos1, CVector3 VPos2);

/////////////////////////////////////// CROSS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This returns a perpendicular vector from 2 given vectors by taking the cross product.
/////
/////////////////////////////////////// CROSS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
												
CVector3 Cross(CVector3 vVector1, CVector3 vVector2);

/////////////////////////////////////// NORMALIZE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This returns a normalize vector (A vector exactly of length 1)
/////
/////////////////////////////////////// NORMALIZE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

CVector3 Normalize(CVector3 vVector);

// This is our 2D point class.  This will be used to store the UV coordinates.
class CVector2 
{
public:
	float x, y;

	CVector2()
	{
		x = 0;
		y = 0;
	}

	CVector2(const CVector2& copy) : x(copy.x), y(copy.y)
	{ // empty		
	}
	
	CVector2(float X, float Y) : x(X), y(Y)
	{ // empty		
	}	

	~CVector2()
	{
		x = 0;
		y = 0;
	}
};


#endif